#ifndef _ORDER_H
#define _ORDER_H

#include <Locker.h>

namespace BExperimental {

class TOrderBase
{
public:
	TOrderBase(size_t size);
	virtual ~TOrderBase();
	
	size_t operator[](int32 i) const;
	
	virtual int Compare(size_t e1, size_t e2) const = 0;

private:
	static BLocker fGlobalLock;
	static TOrderBase* fOrderObj;
	
	static int compare_elements(const void* e1, const void* e2);
	size_t* Order() const;
	
	size_t fSize;
	size_t* fOrder;
};

template<class Container, class Value> class TArrayOrder : public TOrderBase
{
public:
	TArrayOrder(const Container& container, size_t size,
				int (*compare)(const Value& v1, const Value& v2))
		: TOrderBase(size),
		  fContainer(container),
		  fCompare(compare)
	{
	}
	
	virtual ~TArrayOrder()
	{
	}
	
	const Value& ValueAt(int32 i) const
	{
		return fContainer[(*this)[i]];
	}
	
	virtual int Compare(size_t e1, size_t e2) const
	{
		return (*fCompare)(fContainer[e1], fContainer[e2]);
	}

private:
	const Container& fContainer;
	int (*fCompare)(const Value& v1, const Value& v2);
};

};	// namespace BExperimental

using namespace BExperimental;

#endif
