#ifndef _DNS_CACHE_H
#define _DNS_CACHE_H

#include <Locker.h>
#include "Queue.h"

const int kMaxHostNameLength = 255;	// according to the RFC.
const int kDNSCacheSize = 15;	// This is also the limit on concurrent requests
const uint kDNSHashSize = 17;

class DNSCache {
public:
	DNSCache();
	virtual ~DNSCache();
	status_t GetHostByName(const char *hostname, uint *outSockAddr);
	void KillPendingRequests();

private:
	struct DNSCacheEntry : public QueueEntry {
		DNSCacheEntry *fHashNext, **fHashPrev;
		char hostname[kMaxHostNameLength + 1];		
		ulong ip;					// (in network byte order)
		status_t error;
		bigtime_t requestTime;
		bigtime_t expiration;
		int retryCount;
		bool pending;
		int32 requestCount;
	};

	void SendRequest(DNSCacheEntry *entry);
	bool WaitForResponse();
	void SignalEntry(DNSCacheEntry *entry);
	bool IsNumericAddress(const char *addr);
	void Initialize();
	static void TimeoutRequest(int);
	void DumpEntries();

	DNSCacheEntry fCacheEntries[kDNSCacheSize];
	BLocker fLock;
	Queue fLRU;
	DNSCacheEntry *fHash[kDNSHashSize];
	int fSocket;
	bool fWaitingOnResponse;
	class Condition *fCondition;
	bool fShutdown;
	bool fInitialized;

#if PRINT_STATISTICS
	int32 fHits;
	int32 fRequests;
#endif
};

extern DNSCache dnsCache;

#endif
